/* Sigma - Simple greedy multiple alignment */
/* Copyright (C) 2007 Rahul Siddharthan     */
/* Licensed under the GNU GPL, version 2    */
/* See file COPYING in source distribution  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "sigma.h"
#include "commonroutines.h"



int get_transcount_from_str(char *s, int *nuc1, int *nuc2, double *count) 
{
    /* return code 1 = acceptable error (continue),
       2 = unacceptable error */
    int m=0;

    if (strlen(s)==0)
        return 1;
    if (s[0]=='#')
        return 1;
    if (strlen(s) < 3)
        return 2;
    m=0;
    while ((m<strlen(s))&&(m<255)&&((s[m]==' ')||(s[m]=='\t')))
        m++;
    *nuc1 = base2num(s[m]);
    m++;
    if (*nuc1 < 0)
        return 2;
    
    while ((m<strlen(s))&&(m<255)&&((s[m]==' ')||(s[m]=='\t'))) 
        m++;
    
    *nuc2 = base2num(s[m]);
    if (*nuc2 < 0)
        return 2;
    
    m++;

    while ((m<strlen(s))&&(m<255)&&((s[m]==' ')||(s[m]=='\t'))) 
        m++;
    *count = atof((char*)(s+m));

    return 0;
}




int gettransitionrates(params *v)  /* return 1 if illegal char in file, 2 if
                            * incomplete file */
{ 
    FILE *probfile;
    char s[256];
    int m,n,nuc1,nuc2,status;
    double count,tmptrans[16], tot;

    
    probfile=fopen(v->transfilename,"rt");
    if (probfile==NULL)
        return 1;

    for (m=0; m<4; m++) 
        for (n=0; n<4; n++) {
            if (n==m) {
                tmptrans[m*4+n] = 0.0;
                v->transitionmat[m*4+n] = 0.0;
                v->invtransitionmat[m*4+n] = 0.0;
            }
            else
                tmptrans[m*4+n]=-1.0;
        }
    

    while (fgets(s,255,probfile)) {
        status = get_transcount_from_str(s,&nuc1,&nuc2,&count);
        if (status == 1)
            continue;
        if (status)
            return 1;
        if (count <= 0.0)
            return 2;
        tmptrans[nuc1*4+nuc2] = count;
    }

    for (m=0; m<4; m++) {
        tot = 0.0;
        for (n=0; n<4; n++) {
            if (tmptrans[m*4+n]<0.0)
                return 2;
            tot += tmptrans[m*4+n];
        }
        for (n=0; n<4; n++) 
            v->transitionmat[m*4+n] = tmptrans[m*4+n]/tot;
        tot = 0.0;
        for (n=0; n<4; n++) 
            tot += tmptrans[n*4+m];
        for (n=0; n<4; n++) 
            v->invtransitionmat[m*4+n] = tmptrans[n*4+m]/tot;
        
    }
        
    return 0;
}
