/* Sigma - Simple greedy multiple alignment */
/* Copyright (C) 2007 Rahul Siddharthan     */
/* Licensed under the GNU GPL, version 2    */
/* See file COPYING in source distribution  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "header.h"
#include "commonroutines.h"
#include "treer.h"
#include "setbgw.h"


void fixdirs(params *v, GNode *node, int sign) 
{
    GNode *child;
    int i,n;
    
    sign *= ((treest*)(node->data))->dir;
    if (node->children == NULL) {
        i = ((treest*)(node->data))->index;
        n = g_array_index(v->keepseqs,int,i);
        g_array_index(v->seqdirlist,int,n) = sign;
    }
    else {
        child = node->children;
        while (child) {
            fixdirs(v,child,sign);
            child = child->next;
        }
    }
}

void initseqfrags(params *v, GPtrArray *seqs)
{
    int n,nn,len, i;
    fastaseq *oneseq;
    seq_frag *onefrag;
    GString *rseq;

    v->seqfraglist = g_ptr_array_new();
    v->seqheadlist = g_ptr_array_new();
    v->seqdirlist = g_array_new(TRUE,TRUE,sizeof(int));
    i=1;
    for (n=0; n<v->nseqs; n++)
        g_array_append_val(v->seqdirlist,i);

    
    if (v->revcomp) {
        fixdirs(v,v->pTree,1);
        // make sure there are more plusses than minuses
        i = 0;
        for (n=0; n<v->nseqs; n++)
            if (g_array_index(v->seqdirlist,int,n) == -1)
                i++;
        if (i > (v->nseqs)/2)
            for (n=0; n<v->nseqs; n++)
                g_array_index(v->seqdirlist, int, n) *= -1;
    }       
    

    for (n=0; n<v->keepseqs->len; n++) {
        i = g_array_index(v->keepseqs, int, n);

        oneseq= g_ptr_array_index(seqs,i);

        len = (oneseq->seq->len);


        onefrag = new_seqfrag(v);
        onefrag->seqlen = len;
        
        onefrag->nseqs=1;
        onefrag->seqindex[0] = n;        

        if (g_array_index(v->seqdirlist,int,i) == 1)
            onefrag->seq[n] = (oneseq->seq->str);
        else {
            rseq = g_string_new(oneseq->seq->str);
            for (nn=oneseq->seq->len-1; nn>=0; nn--)
                    rseq->str[oneseq->seq->len-nn-1] = revcomp(oneseq->seq->str[nn]);
            onefrag->seq[n] = rseq->str;
        }

        onefrag->likelihood = calloc(onefrag->seqlen, sizeof(double));
        onefrag->transprob[n] = calloc((onefrag->seqlen)*4,sizeof(double));
            
        settransprobs(v,onefrag,NULL,v->pTree); 
        set_likelihoods(v,&onefrag);
        
        onefrag->seqlabel[n] = malloc((4)*sizeof(char));
        onefrag->seqlabel[n][0] = '0';
        onefrag->seqlabel[n][1] = '.';
        onefrag->seqlabel[n][2] = '1';
        onefrag->seqlabel[n][3] = '\0';

        onefrag->neighbours_l[n] = NULL;
        onefrag->neighbours_r[n] = NULL;
        
        g_ptr_array_add((v->seqfraglist),onefrag);
        g_ptr_array_add((v->seqheadlist),onefrag);
        
    }
}

