#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <ctype.h>
#include "header.h"
#include "commonroutines.h"
#include "ptreer.h"
#include "treer.h"
#include "blockseq.h"
#include "initseqfrags.h"
#include "assemble_frags_to_seqs.h"

void getmatch(char * oneseq, char *twoseq, int index1, int index2, int l, int *matches, int* total)
{
    int i, j;
    *total = 0, *matches = 0;
    for (i=index1,j=index2; i<(index1+l) && j<(index2+l); i++,j++)
    {
        if (oneseq[i] == twoseq[j])
            (*matches)++;
        (*total)++;
    }
}


double getbgvalue(params* v)
{   
        
    double ancbgprob[4], totancbgprob;
    int i;

    totancbgprob = 0.0;
    for (i=0;i<4;i++)
    {
        ancbgprob[i] = v->singlesite[i];
        totancbgprob += (ancbgprob[i] * ancbgprob[i]);
    }
    return totancbgprob;
}   



double make_pair_matches(params *v, fastaseq *seq1, fastaseq *seq2)
{
    params *v_temp;
    int n1,n2;
    double total = 0.0, matches = 0.0;
    GPtrArray *seqs, *outarray;
    GString *s1, *s2;
    

    v_temp = calloc(1,sizeof(params));

    v_temp->correlbg = v->correlbg;
    v_temp->pmatch = v->pmatch;
    v_temp->nseqs = 2;
    v_temp->usetransitions = v->usetransitions;
    for (n1=0; n1 < 4; n1++) {
        v_temp->singlesite[n1] = v->singlesite[n1];
        for (n2=0; n2 < 4; n2++) {
            v_temp->dinucleotide_l[n1*4+n2] = v->dinucleotide_l[n1*4+n2];
            v_temp->dinucleotide_r[n1*4+n2] = v->dinucleotide_r[n1*4+n2];
            v_temp->transitionmat[n1*4+n2] = v->transitionmat[n1*4+n2];
            v_temp->invtransitionmat[n1*4+n2] = v->invtransitionmat[n1*4+n2];
        }
    }
    v_temp->keepseqs = g_array_new(TRUE,TRUE,sizeof(int));
    v_temp->ignoreseqs = g_array_new(TRUE,TRUE,sizeof(int));
    for (n1=0; n1<2; n1++)
        g_array_append_val(v_temp->keepseqs,n1);

    seqs = g_ptr_array_new();
    g_ptr_array_add(seqs, seq1);
    g_ptr_array_add(seqs, seq2);
    
    createStarTree(v_temp,seqs,0.3333);

    initseqfrags(v_temp, seqs);
    blockseq(v_temp);
    assemble_frags_to_seqs(v_temp, &outarray, seqs);

    s1 = g_ptr_array_index(outarray,0);
    s2 = g_ptr_array_index(outarray,1);
    for (n1=0; n1<s1->len && n1 < s2->len; n1++) {
        if (isupper(s1->str[n1]) && isupper(s2->str[n1])) {
            total += 1.0;
            if (s1->str[n1]==s2->str[n1])
                matches += 1.0;
        }
    }


    g_ptr_array_free(seqs,FALSE);
    g_ptr_array_free(v_temp->seqfraglist,FALSE);
    g_ptr_array_free(v_temp->seqheadlist,FALSE);
//    g_string_free(s1,TRUE);
//    g_string_free(s2,TRUE);
//    g_ptr_array_free(outarray,TRUE);
    /* FIXME? many leaks remain, but perhaps not worth fixing */

    total = (float) (seq1->seq->len > seq2->seq->len ? seq2->seq->len : seq1->seq->len);
    
    if (total > 0.0)
        return matches/total;
    else
        return 0.0;
}



void getDistanceMatrix(params *v, GPtrArray *seqs, double ***distance) 
{
    int i, j, nn;
    double f, mf, bgvalue;
    fastaseq *seq1, *seq2, *seq3;
    
        
    /* create 2D distance array */
    *distance = (double**)calloc(sizeof(double*), 2*seqs->len*(v->revcomp+1));
    for (i=0;i<2*seqs->len ;i++)
        *((*distance) +i) = (double*)calloc(sizeof(double), 2*seqs->len*(v->revcomp+1));

    bgvalue = getbgvalue(v);

    for (i=0; i < seqs->len - 1; i++) {
        seq1 = g_ptr_array_index(seqs,i);
        for (j = i+1; j < seqs->len; j++) {
            seq2 = g_ptr_array_index(seqs,j);
            f = make_pair_matches(v, seq1, seq2);
            if (f > 0.0) {
                mf = (f-bgvalue)/(1.0-bgvalue);
                //FIXME - q shouldn't be too large
                
                if (mf > 0.99)
                    mf = 0.99;
                if (mf < 0)
                    mf = 0;
                if (v->revcomp) {
                    (*distance)[i*2][j*2] = mf;
                    (*distance)[i*2+1][j*2+1] = mf;
                }
                else
                    (*distance)[i][j] = mf;
            }
            else {
                if (v->revcomp) {
                    (*distance)[i*2][j*2] = 0.0;
                    (*distance)[i*2+1][j*2+1] = 0.0;
                }
                else
                    (*distance)[i][j] = 0;
            }
            if (v->revcomp) {
                (*distance)[j*2][i*2] = (*distance)[i*2][j*2];
                (*distance)[j*2+1][i*2+1] = (*distance)[i*2][j*2];
            }
            else
                (*distance)[j][i] = (*distance)[i][j];
            if (v->revcomp) {
                seq3 = malloc(sizeof(fastaseq));  // FIXME free
                seq3->seq = g_string_new(seq2->seq->str);
                seq3->header = seq2->header;
                for (nn=seq2->seq->len-1; nn>=0; nn--)
                    seq3->seq->str[seq2->seq->len-nn-1] = revcomp(seq2->seq->str[nn]);
                f = make_pair_matches(v, seq1, seq3);
                if (f > 0.0) {
                    mf = (f-bgvalue)/(1.0-bgvalue);
                    //FIXME - q shouldn't be too large
                
                    if (mf > 0.99)
                        mf = 0.99;
                    if (mf < 0)
                        mf = 0;
                    (*distance)[i*2][j*2+1] = mf;
                    (*distance)[i*2+1][j*2] = mf;
                }
                else {
                    (*distance)[i*2+1][j*2] = 0.0;
                    (*distance)[i*2][j*2+1] = 0.0;

                }
                (*distance)[j*2][i*2+1] = (*distance)[i*2][j*2+1];
                (*distance)[j*2+1][i*2] = (*distance)[i*2+1][j*2];
            }
        } /*end of j loop */
        
    } /* end of for i loop */
}



void getDistanceMatrix_final(params *v, GPtrArray *assembled, double ***distance) 
{
    int m,n, n1;
    double matches, total, bgvalue, f, mf, s1len, s2len;
    GString *s1, *s2;

    /* create 2D distance array */
    *distance = (double**)calloc(sizeof(double*), 2*assembled->len);
    
    for (n=0; n<2*assembled->len; n++)
        *((*distance) +n) = (double*)calloc(sizeof(double), 2*assembled->len);
    
    bgvalue = getbgvalue(v);
    for (m=0; m<assembled->len - 1; m++)
        for (n=m+1; n<assembled->len; n++) {
            total = 0.0;
            matches = 0.0;
            s1len = 0.0;
            s2len = 0.0;
            
            
            s1 = g_ptr_array_index(assembled,m);
            s2 = g_ptr_array_index(assembled,n);
            for (n1=0; n1<s1->len && n1 < s2->len; n1++) {
                if (s1->str[n1] != '-')
                    s1len += 1;
                if (s2->str[n1] != '-')
                    s2len += 1;     
                if (isupper(s1->str[n1]) && isupper(s2->str[n1])) {
                    total += 1.0;
                    if (s1->str[n1]==s2->str[n1])
                        matches += 1.0;
                }
            }
            total = (float) (s1len > s2len ? s2len : s1len);
            f = matches/total;
            mf = (f-bgvalue)/(1.0-bgvalue);
            if (mf > 1.0)
                mf = 1.0;
            if (mf < 0.0)
                mf = 0.0;
            (*distance)[m][n] = mf;
            (*distance)[n][m] = mf;
        }
}


void getPTree(params *v, GPtrArray *seqs, GPtrArray *assembled)
{
    double **distance = NULL, **distance_keep;
    int i, j, keep,n1, n2;
    GPtrArray *seqs_keep, *outarray;
    fastaseq *seq1;
    params *v_temp;
    

    /* create distance matrix */
    if (assembled) {
        v->revcomp = 0;
        getDistanceMatrix_final(v,assembled,&distance);

        if (v->startree)
            create_star_phylo_tree(v, seqs, distance);
        else
            create_phylo_tree(v, seqs, distance);
    }
    else {
        if (v->flattree >= 0) {
            createStarTree(v,seqs,v->flattree);
            v->keepseqs = g_array_new(TRUE,TRUE,sizeof(int));
            v->ignoreseqs = g_array_new(TRUE,TRUE,sizeof(int));
            for (i=0; i<v->nseqs; i++)
                g_array_append_val(v->keepseqs,i);
            return;
        }

        v_temp = calloc(1,sizeof(params));

        v_temp->correlbg = v->correlbg;
        v_temp->pmatch = v->pmatch;
        v_temp->nseqs = v->nseqs;
        v_temp->usetransitions = v->usetransitions;
        for (n1=0; n1 < 4; n1++) {
            v_temp->singlesite[n1] = v->singlesite[n1];
            for (n2=0; n2 < 4; n2++) {
                v_temp->dinucleotide_l[n1*4+n2] = v->dinucleotide_l[n1*4+n2];
                v_temp->dinucleotide_r[n1*4+n2] = v->dinucleotide_r[n1*4+n2];
                v_temp->transitionmat[n1*4+n2] = v->transitionmat[n1*4+n2];
                v_temp->invtransitionmat[n1*4+n2] = v->invtransitionmat[n1*4+n2];
            }
        }
        v_temp->keepseqs = g_array_new(TRUE,TRUE,sizeof(int));
        v_temp->ignoreseqs = g_array_new(TRUE,TRUE,sizeof(int));
        for (n1=0; n1<v->nseqs; n1++)
            g_array_append_val(v_temp->keepseqs,n1);

        createStarTree(v_temp,seqs,0.3333);

        initseqfrags(v_temp, seqs);
        blockseq(v_temp);
        assemble_frags_to_seqs(v_temp, &outarray, seqs);

        
        getDistanceMatrix_final(v,outarray,&distance);

        
        /* remap seqs */
        v->keepseqs = g_array_new(TRUE,TRUE,sizeof(int));
        v->ignoreseqs = g_array_new(TRUE,TRUE,sizeof(int));

        for (i=0; i < seqs->len; i++) {
            keep = 0;
            for (j=0; j < seqs->len; j++)
                if (v->revcomp) {
                    if ((j!=i) && ((distance[2*i][2*j]>0.001)
                                   ||(distance[2*i][2*j+1]>0.001))) {
                        keep=1;
                        break;
                    }
                }
                else {
                    if ((j!=i) && (distance[i][j]>0.001)) {
                        keep = 1;
                        break;
                    }
                }
            
            
            if (keep)
                g_array_append_val(v->keepseqs,i);
            else
                g_array_append_val(v->ignoreseqs,i);
        }
        if (v->keepseqs->len == 0) {
            v->pTree=NULL;
            return;
        }
    
    
        seqs_keep = g_ptr_array_new();
        for (i=0; i<v->keepseqs->len; i++) {
            seq1 = g_ptr_array_index(seqs,g_array_index(v->keepseqs,int,i));
            g_ptr_array_add(seqs_keep,seq1);
        }

        distance_keep = calloc(2*(v->revcomp+1)*v->keepseqs->len,
                               sizeof(double*));
        for (i=0; i<2*(v->revcomp+1)*v->keepseqs->len; i++)
            distance_keep[i] = calloc(2*(v->revcomp+1)*v->keepseqs->len,
                                      sizeof(double));
        for (i=0; i<v->keepseqs->len; i++)
            for (j=0; j<v->keepseqs->len; j++) {
                if (v->revcomp){
                    distance_keep[2*i][2*j] = distance[2*g_array_index(v->keepseqs,int,i)][2*g_array_index(v->keepseqs,int,j)];
                    distance_keep[2*i+1][2*j] = distance[2*g_array_index(v->keepseqs,int,i)+1][2*g_array_index(v->keepseqs,int,j)];
                    distance_keep[2*i][2*j+1] = distance[2*g_array_index(v->keepseqs,int,i)][2*g_array_index(v->keepseqs,int,j)+1];
                    distance_keep[2*i+1][2*j+1] = distance[2*g_array_index(v->keepseqs,int,i)+1][2*g_array_index(v->keepseqs,int,j)+1];
                }
                else
                    distance_keep[i][j] = distance[g_array_index(v->keepseqs,int,i)][g_array_index(v->keepseqs,int,j)];
            }
    
    
        if (v->startree)
            create_star_phylo_tree(v, seqs_keep, distance_keep);
        else
            create_phylo_tree(v, seqs_keep, distance_keep);
        
        for (i=0; i<2*(v->revcomp+1)*v->keepseqs->len ;i++) 
            free(distance_keep[i]);
        
        free(distance_keep);


//        printTreeNewick(stdout, v->pTree);
//        printf("\n");
    }
    

    for (i=0;i<2*seqs->len ;i++) 
        free(distance[i]);
    
    free(distance);

    /* FIXME : more frees required */
}
