/* Sigma - Simple greedy multiple alignment */
/* Copyright (C) 2007 Rahul Siddharthan     */
/* Licensed under the GNU GPL, version 2    */
/* See file COPYING in source distribution  */

#include "sigma.h"
#include "fasta.h"
#include "commonroutines.h"

int setbgprobs(params *v, GPtrArray *bgseqs) 
{
    int i,p,m,n,n1,n2;
    fastaseq *fseq;
    double stot,dtot;
   
    for (m=0; m<4; m++) {
        v->singlesite[m]=0.0;
        for (n=0; n<4; n++) {
            v->dinucleotide_l[m*4+n]=0.0;
            v->dinucleotide_r[m*4+n]=0.0;
        }
    }

    if (v->correlbg == 0) {
        for (m=0; m<4; m++) {
            v->singlesite[m] = 0.25;
            for (n=0; n<4; n++) {
                v->dinucleotide_l[m*4+n] = 0.25;
                v->dinucleotide_r[m*4+n] = 0.25;
            }
        }
        return 0;
    }

    for (i=0; i<bgseqs->len; i++) {
        fseq = g_ptr_array_index(bgseqs,i);
        for (p=0; p<fseq->seq->len-1; p++) {
            if (base2num(fseq->seq->str[p])<0)
                continue;
            v->singlesite[base2num(fseq->seq->str[p])] += 1.0;
            if (base2num(fseq->seq->str[p+1])<0)
                continue;
            n1 = base2num(fseq->seq->str[p]);
            n2 = base2num(fseq->seq->str[p+1]);
            v->dinucleotide_l[n2*4+n1] += 1.0;
            v->dinucleotide_r[n1*4+n2] += 1.0;
        }
    }

    stot=0.0;
    for (m=0; m<4; m++) {
        stot += v->singlesite[m];
        dtot=0.0;
        for (n=0; n<4; n++)
            dtot += v->dinucleotide_l[n*4+m];
        for (n=0; n<4; n++)
            v->dinucleotide_l[n*4+m] /= dtot;
        dtot=0.0;
        for (n=0; n<4; n++)
            dtot += v->dinucleotide_r[n*4+m];
        for (n=0; n<4; n++)
            v->dinucleotide_r[n*4+m] /= dtot;
    }
    for (m=0; m<4; m++)
        v->singlesite[m] /= stot;

    return 0;
}
