 #include <stdlib.h>
 #include <stdio.h>
 #include <assert.h>
 #include <gsl/gsl_math.h>
 #include "header.h"
 #include "treer.h"
 #include "commonroutines.h"

 int distlen = 0;

 char base[4] = {'A', 'C', 'G', 'T'};

 void printTreeNewick(FILE*stream, GNode*treenode)
 {
     GNode *child;
     treest* tst;

     if (treenode == NULL)
             return;
     fprintf(stream,"(");
     child=treenode->children;
     while (child != NULL) {
     tst = (treest*)child->data;
             if (child->children != NULL) {
                 printTreeNewick(stream,child);
                 if (tst->depth !=0)fprintf(stream, ":%0.04f", tst->depth);
             }
             else
             {
                 fprintf(stream,"%d", tst->index);
                 fprintf(stream, ":%0.04f", tst->depth);
             }
             child = child->next;
             if (child!= NULL)
                 fprintf(stream, ",");
     }
     fprintf(stream,")");
 }



 gboolean  printNode(GNode *node, void *Data)
 {
     treest * treedata;

     if (node->data!= NULL)
     {
         printf("\n");
         treedata = (treest*)node->data;
         printf("\nHeader = %s", treedata->seq->header->str);
         printf("\nNum Seqs = %d", treedata->numofseqs);
         printf("\nDepth of node = %f, index =%d", treedata->depth,treedata->index);
     }
     return 0;
 } /* end of printNode */


 void findmaxdist(double *max,  int *maxi, int *maxj, double **distance)
 {
     int i =0, j=0;
     for (i=0;i<distlen;i++)
     {
         for (j=i+1;j<distlen;j++)
         {
             if ((distance[i][j]) > *max) 
             {
                     *max= distance[i][j]; *maxi = i;*maxj=j;
             }
         }
      }
 }

 void printdist(int range, double **distance)
 {
     int i =0, j=0;

     for (i=0;i<range;i++)
     {
         for (j=0;j<range;j++) {
             printf("\t%0.04f", distance[i][j]); 
         }
         printf("\n");
     }
 }

 void printT(GNode *gParent)
 {
     GNode* gNode = NULL;
     treest *treestr= NULL;
     gNode= gParent->children;
     while (gNode != NULL)
     {
         if (gNode->children != NULL)
         {
             printT(gNode);
         }
         treestr = (treest*)gNode->data;
         printf("\n %s , %0.04f", treestr->seq->header->str, treestr->depth);
         gNode = gNode->next;

     } /* end of while */
     printf("\n");
 }
 void printTree(GNode * pT)
 {
     printT(pT);
 }

 int getNumOfChildren(GNode * gN)
 {
     GNode * gNode = NULL;
     gNode = gN->children;
     int count = 0;
     while (gNode != NULL)
     {
       count ++;
       gNode = gNode->next;
     }
     return count;
 }


 void getClosestAncestor(params *v, GNode **gAncestor, int *seqindex,  int nseqs)
 {
     int m =0;
     getClosestAncestorRtn(v->pTree, gAncestor, seqindex, nseqs, &m);
 }



 void getClosestAncestorRtn(GNode * gParent, GNode **gAncestor, int *seqindex,  int nseqs, int *pNumAdded)
 {
     /* strategy: if not all seqs in seqindex are under gParent, return NULL;
      * otherwise if any child includes all seqs in seqindex, return that
      * child; else return self
      */

     int nseqs_included, nseqs_in_node1, i;
     GNode *node1, *return_node;
     treest *st;

     nseqs_included = 0;
     node1 = gParent->children;
     while (node1) {
         if (node1->children) {
             nseqs_in_node1 = 0;
             return_node = NULL;

             getClosestAncestorRtn(node1,&return_node,seqindex,nseqs,&nseqs_in_node1);
             if (return_node) {
                 *pNumAdded = nseqs;
                 *gAncestor = return_node;
                 return;
             }
             else if (nseqs_in_node1 == nseqs) {
                 *pNumAdded = nseqs;
                 *gAncestor = node1;
                 return;
             }
             else
                 nseqs_included += nseqs_in_node1;
         }
         else {
             st = node1->data;
             for (i=0;i<nseqs;i++)
                 if (seqindex[i] == st->index)
                     nseqs_included++;
         }

         node1 = node1->next;
     }
     if (nseqs_included < nseqs)
         *gAncestor = NULL;
     else
         *gAncestor = gParent;
     *pNumAdded = nseqs_included;
 }




 void getSubTree(params *v, GNode **pSubTreeParent, int *seqIndex, int numSeqs, GNode **pClosestAnc)
 {
     int m =0, n=0;
     getSubTreeRtn(v, v->pTree, pSubTreeParent, seqIndex, numSeqs, pClosestAnc, &m, &n);

     /* add leaves to this subtree, so that we won't have to do it repeatedly
      * in getTreeProb*/

 //    add_leaves_to_subtree(v, pSubTreeParent);


 }


 void getSubTreeRtn(params *v, GNode *gParent, GNode **pSubTreeParent, int *seqIndex, 
              int numSeqs, GNode **pClosestAnc, int* pNumAdded, int * pCA)
 {
     GNode* gNode = NULL, *pSubTreeNode = NULL;
     treest *treestr= NULL;
     int i =0;
     treest *treedata = NULL;
     gNode= gParent->children;
     int position = 0;
     int prevNum;

     prevNum = *pNumAdded; int prevCA = *pCA;
     while (gNode != NULL)
     {
         pSubTreeNode = NULL;
         treedata = (treest*)gNode->data;
         if (gNode->children != NULL)
         {

             getSubTreeRtn(v, gNode, &pSubTreeNode, seqIndex, numSeqs, 
                         pClosestAnc, pNumAdded, pCA);
             if ((*pNumAdded == numSeqs) && (*pCA == numSeqs)) {
                 if ((*pSubTreeParent != NULL) && (pSubTreeNode != NULL))
                     g_node_insert(*pSubTreeParent, position, pSubTreeNode);
                 else if (*pSubTreeParent == NULL)
                     *pSubTreeParent = pSubTreeNode;
                 return;
             }
             if (pSubTreeNode != NULL){
                 int numOfChildren = getNumOfChildren(pSubTreeNode);

                 if (gParent != v->pTree){
                     if (*pSubTreeParent == NULL){
                         /* Create a new parent and append the pSubTreeNode */
                         treestr = (treest*)gParent->data;
                         treedata = (treest*)calloc(sizeof(treest), 1);
                         treedata->seq = treestr->seq;
                         treedata->numofseqs = treestr->numofseqs;
                         treedata->depth = treestr->depth;
                         treedata->index = treestr->index;
                         *pSubTreeParent = g_node_new(treedata);
                    }
                    if (numOfChildren == 1)
                    {
                        treestr = (treest*)(pSubTreeNode->children)->data;
                        GNode *temp = pSubTreeNode->children;
                        treedata = (treest*)pSubTreeNode->data;
                        treestr->depth *= treedata->depth;
                        pSubTreeNode = g_node_new(treestr);
                        pSubTreeNode->children = temp->children;

                    }

                    g_node_insert(*pSubTreeParent, position, pSubTreeNode); 
                    position++;
                    if (gParent == *pClosestAnc)
                    {
                         (*pCA)++;
                    }
                 }
                 else
                   *pSubTreeParent = pSubTreeNode;
              }
         }
         else
         {
             treestr = (treest*)gNode->data;
             int pos = 0;
             for (i=0;i<numSeqs;i++)
             {
                 if (seqIndex[i] == treestr->index){
                      if (*pSubTreeParent == NULL){
                          /* create the parent node */
                          if (gParent != v->pTree){
                          treestr = (treest*)gParent->data;
                          treedata = (treest*)calloc(sizeof(treest), 1);
                          treedata->seq = treestr->seq;
                          treedata->numofseqs = treestr->numofseqs;
                          treedata->depth = treestr->depth;
                          treedata->index = treestr->index;
                     }
                     else{
                          treedata = (treest*)calloc(sizeof(treest), 1);
                          treedata->seq = NULL;
                          treedata->numofseqs = 1;
                          treedata->depth = 0;
                          treedata->index = 0;
                     }
                     *pSubTreeParent = g_node_new(treedata);
                 }

                 treestr = (treest*)gNode->data;
                 treedata = (treest*)calloc(sizeof(treest), 1);
                 treedata->seq = treestr->seq;
                 treedata->numofseqs = treestr->numofseqs;
                 treedata->depth = treestr->depth;
                 treedata->index = treestr->index;
                 pSubTreeNode = g_node_new(treedata);

                 g_node_insert(*pSubTreeParent, pos, pSubTreeNode);
                 pos++;
                 if (gParent == *pClosestAnc)
                 {
                     (*pCA)++;
                 }
                 (*pNumAdded) ++;
               } 
           }
           if  ((*pCA == prevCA) && (*pNumAdded - prevNum > 1))
                 *pCA += (*pNumAdded - prevNum - 1);
        }
        gNode = gNode->next; 
     } /* end of while */
 }


 void initTree(params *v, GPtrArray *seqs,  double ** distance)
 {
     int i =0;
     GNode *gNode = NULL;
     treest *treedata = NULL;
     GNode *gNodePrev = NULL;
     /* Node for the root of the tree */
     v->pTree = g_node_new(NULL); // Creates the root of the tree

     distlen = seqs->len;
     /* Assign all the clusters seperately */
     for (i=0;i<seqs->len;i++)
     {
         treedata = (treest*)calloc(sizeof(treest), 1);
         treedata->seq = g_ptr_array_index(seqs, i);
         treedata->numofseqs = 1;
         treedata->depth = 0;
         treedata->index = i;
         gNode = g_node_new(treedata);
         if (gNodePrev == NULL)
             v->pTree->children = gNode;
         else
             gNodePrev->next = gNode;
         gNodePrev = gNode;
     } /* end of for loop */
     gNode->next = NULL;
 }


 GPtrArray* init_nodes(GPtrArray *seqs) 
 {
     GPtrArray *nodelist;
     GNode *onenode;
     treest *treedata;
     int i;

     nodelist = g_ptr_array_new();
     for (i=0; i<seqs->len; i++) {
         treedata = (treest*) calloc(1, sizeof(treest));
         treedata->seq = g_ptr_array_index(seqs,i);
         treedata->index = i;
         treedata->dir = 1;
         onenode = g_node_new(treedata);
         g_ptr_array_add(nodelist, onenode);
     }
     return nodelist;
 }



 void merge_two_nodes(GPtrArray **nodelist, double **distance, int maxindex) 
 {
     /* Find closest nodes, and join them */
     /* If joining two leaves: just join them, assign the "q" as squareroot
      * of their pairwise distance
      * If joining leaf to node, or node to node, use formula to calculate
      * q and to "rebalance" the children of the node(s) being joined
      */

     int m,n, m_best, n_best, index1, index2, index1a, index1b, index2a, index2b;
     GNode *node1, *node2, *newnode, *child1a, *child1b, *child2a, *child2b;
     double q1a,q1b,q2a,q2b,q1a2, q1b2, q1a2a, q1a2b, q1b2a, q1b2b, q1a1b, q2a2b;
     double bestdist, newdist;
     treest *newdata;

     m_best = -1;
     n_best = -1;
     bestdist = 0.0;
     for (m=0; m<(*nodelist)->len-1; m++)
         for (n=m+1; n<(*nodelist)->len; n++) {
             node1 = g_ptr_array_index((*nodelist),m);
             node2 = g_ptr_array_index((*nodelist),n);
             index1 = ((treest*)(node1->data))->index;
             index2 = ((treest*)(node2->data))->index;
             if (distance[index1][index2] >= bestdist) {
                 bestdist = distance[index1][index2];
                 n_best = n;
                 m_best = m;
             }
         }
     assert(bestdist > 0.0);

     node1 = g_ptr_array_index((*nodelist),m_best);
     node2 = g_ptr_array_index((*nodelist),n_best);
     index1 = ((treest*)(node1->data))->index;
     index2 = ((treest*)(node2->data))->index;

     newdata = calloc(1,sizeof(treest));
     newdata->index = maxindex;
     newdata->depth = 0.0;


     if ((node1->children==NULL) && (node2->children==NULL)) {
         /* fuse these nodes and be done with it... */
         assert(bestdist > 0);
         ((treest *)(node1->data))->depth = sqrt(bestdist);
         ((treest *)(node2->data))->depth = sqrt(bestdist);
     }

     else if ((node1->children == NULL) || (node2->children == NULL)) {
         if (node1->children == NULL) {
             /* reassign so that node2 is a leaf*/
             newnode = node1;
             node1 = node2;
             node2 = newnode;
             index1 = ((treest*)(node1->data))->index;
             index2 = ((treest*)(node2->data))->index;   
         }
         child1a = node1->children;
         child1b = child1a->next;
         index1a = ((treest *)(child1a->data))->index;
         index1b = ((treest *)(child1b->data))->index;
         q1a1b = distance[index1a][index1b];
         q1a2 = distance[index1a][index2];
         q1b2 = distance[index1b][index2];

         q1a  = sqrt(q1a1b*q1a2/q1b2);
         if (q1a > 0.99) {
             q1a = 0.99;
             q1b = q1a1b/0.99;
         }
         else {
             q1b = sqrt(q1a1b*q1b2/q1a2);
             if (q1b > 0.99) {
                 q1b = 0.99;
                 q1a = q1a1b/0.99;
             }
         }
         ((treest*)(child1a->data))->depth = q1a;
         ((treest*)(child1b->data))->depth = q1b;        

         bestdist = sqrt(q1a2/(((treest*)(child1a->data))->depth)
                         *q1b2/(((treest*)(child1b->data))->depth));
         assert(bestdist < 1.0);
         if (bestdist < 0.001)
             bestdist = 0.001;

 //        printf("bestdist %f     %d %d = %f    %d %d = %f\n",bestdist,index1a,index1,distance[index1a][index1],index1b,index1,distance[index1b][index1]);
         ((treest*)(node1->data))->depth = sqrt(bestdist);
         ((treest*)(node2->data))->depth = sqrt(bestdist);
     }

     else { /* both children are nodes */
         child1a = node1->children;
         child1b = child1a->next;
         child2a = node2->children;
         child2b = child2a->next;
         index1a = ((treest *)(child1a->data))->index;
         index1b = ((treest *)(child1b->data))->index;
         index2a = ((treest *)(child2a->data))->index;
         index2b = ((treest *)(child2b->data))->index;
         q1a1b = distance[index1a][index1b];
         q2a2b = distance[index2a][index2b];        
         q1a2a = distance[index1a][index2a];        
         q1a2b = distance[index1a][index2b];        
         q1b2a = distance[index1b][index2a];        
         q1b2b = distance[index1b][index2b];

         q1a = sqrt(q1a1b*sqrt(q1a2a*q1a2b/q1b2a/q1b2b));
         q1b = sqrt(q1a1b*sqrt(q1b2a*q1b2b/q1a2a/q1a2b));
         q2a = sqrt(q2a2b*sqrt(q1a2a*q1b2a/q1a2b/q1b2b));
         q2b = sqrt(q2a2b*sqrt(q1a2b*q1b2b/q1a2a/q1b2a));
         printf("%f %f %f %f\n",q1a,q1b,q2a,q2b);

         if (q1a > 0.99) {
             q1a = 0.99;
             q1b = q1a1b/q1a;
         }
         else if (q1b > 0.99) {
             q1b = 0.99;
             q1a = q1a1b/q1b;
         }
         if (q2a > 0.99) {
             q2a = 0.99;
             q2b = q2a2b/q2a;
         }
         else if (q2b > 0.99) {
             q2b = 0.99;
             q2a = q2a2b/q2b;
         }

         ((treest*)(child1a->data))->depth = q1a;
         ((treest*)(child1b->data))->depth = q1b;
         ((treest*)(child2a->data))->depth = q2a;
         ((treest*)(child2b->data))->depth = q2b;
         bestdist = sqrt(sqrt(q1a2a*q1a2b*q1b2a*q1b2b)/q1a1b/q2a2b);
         assert(bestdist < 1.0);
         if (bestdist < 0.001)
             bestdist = 0.001;

         ((treest*)(node1->data))->depth = sqrt(bestdist);
         ((treest*)(node2->data))->depth = sqrt(bestdist);

     }

     newnode = g_node_new(newdata);       
     g_node_append(newnode,node1);
     g_node_append(newnode,node2);
     g_ptr_array_remove((*nodelist),node1);
     g_ptr_array_remove((*nodelist),node2);
     g_ptr_array_add((*nodelist),newnode);

     printTreeNewick(stdout,newnode);
     printf("\n");


     /* recalculate distances from new node to all other nodes
      * and insert in distance matrix */

     if ((*nodelist)->len > 1) {
         for (n=0; n<(*nodelist)->len; n++) {
             node1 = g_ptr_array_index((*nodelist),n);
             if (node1==newnode)
                 continue;
             index1 = ((treest *)(node1->data))->index;
             child1a = newnode->children;
             child1b = child1a->next;
             newdist = distance[((treest*)(child1a->data))->index][index1]/((treest*)(child1a->data))->depth;
             newdist *= distance[((treest*)(child1b->data))->index][index1]/((treest*)(child1b->data))->depth;
             newdist = sqrt(newdist);
 //            printf("%f %d %d\n",newdist,index1,maxindex);
             if (newdist > 0.99)
                 newdist = 0.99; // FIXME fix this case
             if (newdist < 0.01)
                 newdist = 0.01;
 //            printf("%f %d %d\n",newdist,index1,maxindex);
             distance[index1][maxindex] = newdist;
             distance[maxindex][index1] = newdist;
         }
         distance[maxindex][maxindex] = 0.0;
     }

     return;

 }




void merge_two_nodes_simple(GPtrArray **nodelist, double **distance, int maxindex, int revcomp) 
 {
     /* Find closest nodes, and join them */
     /* If joining two leaves: just join them, assign the "q" as squareroot
      * of their pairwise distance
      * If joining leaf to node, or node to node, use simple formula
      */

     int m,n, m_best, n_best, index1, index2, index1a, index1b, index2a, index2b;
     int dir1a, dir1b, dir2, dir2a, dir2b;
     GNode *node1, *node2, *newnode, *child1a, *child1b, *child2a, *child2b;
     double q1a,q2a,q1a2, q1b2, q1a2a, q1a2b, q1b2a, q1b2b;
     double bestdist, newdist;
     treest *newdata;

     m_best = -1;
     n_best = -1;
     bestdist = 0.0;
     for (m=0; m<(*nodelist)->len-1; m++)
         for (n=m+1; n<(*nodelist)->len; n++) {
             node1 = g_ptr_array_index((*nodelist),m);
             node2 = g_ptr_array_index((*nodelist),n);
             index1 = ((treest*)(node1->data))->index;
             index2 = ((treest*)(node2->data))->index;
             if (revcomp) {
                 if (distance[index1*2][index2*2] >= bestdist) {
                     bestdist = distance[index1*2][index2*2];
                     n_best = n;
                     m_best = m;
                 }
                 if (distance[index1*2][index2*2+1] >= bestdist) {
                     bestdist = distance[index1*2][index2*2+1];
                     n_best = -n;
                     m_best = m;
                 }
             }
             else {
                 if (distance[index1][index2] >= bestdist) {
                     bestdist = distance[index1][index2];
                     n_best = n;
                     m_best = m;
                 }
             }
         }
     assert(bestdist >= 0.0);

     node1 = g_ptr_array_index((*nodelist),m_best);
     if ((revcomp) && (n_best < 0)) {
         node2 = g_ptr_array_index((*nodelist),-n_best);
         ((treest*)(node2->data))->dir = -1;
     }
     else
         node2 = g_ptr_array_index((*nodelist), n_best);
             
     index1 = ((treest*)(node1->data))->index;
     index2 = ((treest*)(node2->data))->index;

     newdata = calloc(1,sizeof(treest));
     newdata->index = maxindex;
     newdata->depth = 0.0;
     newdata->dir = 1;


     if ((node1->children==NULL) && (node2->children==NULL)) {
         /* fuse these nodes and be done with it... */
         ((treest *)(node1->data))->depth = sqrt(bestdist);
         ((treest *)(node2->data))->depth = sqrt(bestdist);
     }

     else if ((node1->children == NULL) || (node2->children == NULL)) {
         if (node1->children == NULL) {
             /* reassign so that node2 is a leaf*/
             newnode = node1;
             node1 = node2;
             node2 = newnode;
             index1 = ((treest*)(node1->data))->index;
             index2 = ((treest*)(node2->data))->index;
             if (((treest*)(node1->data))->dir == -1) {
                 assert( ((treest*)(node2->data))->dir = 1);
                 ((treest*)(node1->data))->dir = 1;                 
                 ((treest*)(node2->data))->dir = -1;
             }
         }
         child1a = node1->children;
         child1b = child1a->next;
         index1a = ((treest *)(child1a->data))->index;
         index1b = ((treest *)(child1b->data))->index;


         if (revcomp){
             dir1a = ((treest *)(node1->data))->dir * ((treest*)(child1a->data))->dir;
             dir1b = ((treest *)(node1->data))->dir * ((treest*)(child1b->data))->dir;
             dir2 = ((treest*)(node2->data))->dir;

             if (dir1a != dir2) 
                 q1a2 = distance[index1a*2+1][index2*2];
             else
                 q1a2 = distance[index1a*2][index2*2];
             
             if (dir1b != dir2)
                 q1b2 = distance[index1b*2+1][index2*2];
             else 
                 q1b2 = distance[index1b*2][index2*2];

         }
         else {
             q1a2 = distance[index1a][index2];
             q1b2 = distance[index1b][index2];
         }

         // depth of a and b are stored already in data, don't change
         q1a = ((treest*)(child1a->data))->depth;

         bestdist = sqrt(sqrt(q1a2*q1b2)/q1a);
         if (bestdist > 0.99)
             bestdist = 0.99;
         if (bestdist < 0.01)
             bestdist = 0.01;
         
         ((treest*)(node1->data))->depth = bestdist;
         ((treest*)(node2->data))->depth = bestdist;
     }

     else { /* both children are nodes */
         child1a = node1->children;
         child1b = child1a->next;
         child2a = node2->children;
         child2b = child2a->next;
         index1a = ((treest *)(child1a->data))->index;
         index1b = ((treest *)(child1b->data))->index;
         index2a = ((treest *)(child2a->data))->index;
         index2b = ((treest *)(child2b->data))->index;

         q1a = ((treest*)(child1a->data))->depth;         
         q2a = ((treest*)(child2a->data))->depth;         

         if (revcomp){
             dir1a = ((treest *)(node1->data))->dir * ((treest*)(child1a->data))->dir;
             dir1b = ((treest *)(node1->data))->dir * ((treest*)(child1b->data))->dir;
             dir2a = ((treest *)(node2->data))->dir * ((treest*)(child2a->data))->dir;
             dir2b = ((treest *)(node2->data))->dir * ((treest*)(child2b->data))->dir;
             if (dir1a != dir2a)
                 q1a2a = distance[index1a*2+1][index2a*2];
             else
                 q1a2a = distance[index1a*2][index2a*2];

             if (dir1a != dir2b)
                 q1a2b = distance[index1a*2+1][index2b*2];
             else
                 q1a2b = distance[index1a*2][index2b*2];

             if (dir1b != dir2a)
                 q1b2a = distance[index1b*2+1][index2a*2];
             else
                 q1b2a = distance[index1b*2][index2a*2];

             if (dir1b != dir2b)
                 q1b2b = distance[index1b*2+1][index2b*2];
             else
                 q1b2b = distance[index1b*2][index2b*2];
         }
         else {
             q1a2a = distance[index1a][index2a];        
             q1a2b = distance[index1a][index2b];        
             q1b2a = distance[index1b][index2a];        
             q1b2b = distance[index1b][index2b];
         }
         

         bestdist = sqrt(sqrt(sqrt(q1a2a*q1b2b*q1b2a*q1b2b))/q1a/q2a);

         if (bestdist > 0.99)
             bestdist = 0.99;
         if (bestdist < 0.01)
             bestdist = 0.01;

         ((treest*)(node1->data))->depth = bestdist;
         ((treest*)(node2->data))->depth = bestdist;

     }

     newnode = g_node_new(newdata);       
     g_node_append(newnode,node1);
     g_node_append(newnode,node2);
     g_ptr_array_remove((*nodelist),node1);
     g_ptr_array_remove((*nodelist),node2);
     g_ptr_array_add((*nodelist),newnode);

//     printTreeNewick(stdout,newnode);
//     printf("\n");


     /* recalculate distances from new node to all other nodes
      * and insert in distance matrix */

     if ((*nodelist)->len > 1) {
         for (n=0; n<(*nodelist)->len; n++) {
             node1 = g_ptr_array_index((*nodelist),n);
             if (node1==newnode)
                 continue;
             index1 = ((treest *)(node1->data))->index;
             child1a = newnode->children;
             child1b = child1a->next;
             if (revcomp) {
                 newdist = distance[2*((treest*)(child1a->data))->index][2*index1]/((treest*)(child1a->data))->depth;
                 if (((treest*)(child1b->data))->dir == 1) 
                     newdist *= distance[2*((treest*)(child1b->data))->index][2*index1]/((treest*)(child1b->data))->depth;
                 else
                     newdist *= distance[(2*((treest*)(child1b->data))->index)+1][2*index1]/((treest*)(child1b->data))->depth;
                 newdist = sqrt(newdist);
                 //            printf("%f %d %d\n",newdist,index1,maxindex);
                 if (newdist > 0.99)
                     newdist = 0.99; // FIXME fix this case
                 if (newdist < 0.01)
                     newdist = 0.01;
                 //            printf("%f %d %d\n",newdist,index1,maxindex);
                 distance[2*index1][2*maxindex] = newdist;
                 distance[2*maxindex][2*index1] = newdist;
                 distance[2*index1+1][2*maxindex+1] = newdist;
                 distance[2*maxindex+1][2*index1+1] = newdist;
                 double newdist1 = newdist;
                 
                 newdist = distance[(2*((treest*)(child1a->data))->index)+1][2*index1]/((treest*)(child1a->data))->depth;
                 if (((treest*)(child1b->data))->dir == 1) 
                     newdist *= distance[(2*((treest*)(child1b->data))->index)+1][2*index1]/((treest*)(child1b->data))->depth;
                 else
                     newdist *= distance[2*((treest*)(child1b->data))->index][2*index1]/((treest*)(child1b->data))->depth;
                 newdist = sqrt(newdist);
                 //            printf("%f %d %d\n",newdist,index1,maxindex);
                 if (newdist > 0.99)
                     newdist = 0.99; // FIXME fix this case
                 if (newdist < 0.01)
                     newdist = 0.01;
                 //            printf("%f %d %d\n",newdist,index1,maxindex);
                 distance[2*index1][2*maxindex+1] = newdist;
                 distance[2*maxindex+1][2*index1] = newdist;
                 distance[2*index1+1][2*maxindex] = newdist;
                 distance[2*maxindex][2*index1+1] = newdist;
             }
             else {
                 newdist = distance[((treest*)(child1a->data))->index][index1]/((treest*)(child1a->data))->depth;
                 newdist *= distance[((treest*)(child1b->data))->index][index1]/((treest*)(child1b->data))->depth;
                 newdist = sqrt(newdist);
                 //            printf("%f %d %d\n",newdist,index1,maxindex);
                 if (newdist > 0.99)
                     newdist = 0.99; // FIXME fix this case
                 if (newdist < 0.01)
                     newdist = 0.01;
                 //            printf("%f %d %d\n",newdist,index1,maxindex);
                 distance[index1][maxindex] = newdist;
                 distance[maxindex][index1] = newdist;
             }
         }
         if (revcomp) {
             distance[2*maxindex][2*maxindex] = 0.0;
             distance[2*maxindex][2*maxindex+1] = 0.0;
             distance[2*maxindex+1][2*maxindex] = 0.0;
             distance[2*maxindex+1][2*maxindex+1] = 0.0;
         }
         else
             distance[maxindex][maxindex] = 0.0;
     }

     return;

 }



void create_phylo_tree(params *v, GPtrArray *seqs, double **distance) 
 {
     GPtrArray *nodelist;
     int maxindex;
     GNode *rootnode;

     nodelist = init_nodes(seqs);

     maxindex = seqs->len;

     while (nodelist->len > 1) {
         merge_two_nodes_simple(&nodelist,distance,maxindex,v->revcomp);
         maxindex += 1;
     }

     rootnode = g_ptr_array_index(nodelist,0);
     v->pTree = rootnode;
     g_ptr_array_free(nodelist,TRUE);
 }



void create_star_phylo_tree(params *v, GPtrArray *seqs, double **distance) 
{
    /* this is supposed to make a good star tree from the distance matrix
     * 
     * algorithm:
     *
     * 1. assign each seq a score which is the geometric mean of its
     * distance (proximity) from all other nodes, where for each other
     * node, the more favourable of the two orientations is taken
     *
     * 2. going by highest to lowest score, add each seq to the tree
     * with a depth equal to the square root of its score, and with an
     * orientation that favours the sequences that were already
     * added to the tree
     *
     */

    int n,m,m1,bestm;
    double dist,dist1,dist1r,bestdist;
    GArray *scores, *dirs, *added;
    GNode *root, *child;
    treest *newdata;
    
    newdata = calloc(1,sizeof(treest));
    newdata->index = -1;
    newdata->depth = 0.0;
    newdata->dir = 1;
    root = g_node_new(newdata);

    scores = g_array_new(TRUE,TRUE,sizeof(double));
    dirs = g_array_new(TRUE,TRUE,sizeof(int));
    added = g_array_new(TRUE,TRUE,sizeof(int)); 
    
    for (n=0; n<seqs->len; n++) {
        dist = 1.0;
        for (m=0; m<seqs->len; m++) {
            if (m==n)
                continue;
            if (v->revcomp) {
                dist1 = distance[n*2][m*2];
                dist1r = distance[n*2][m*2+1];
                if (dist1 > dist1r)
                    dist *=dist1;
                else
                    dist *= dist1r;
            }
            else
                dist *= distance[n][m];
        }
        dist = exp(log(dist)/((float)(seqs->len - 1.0)));
        g_array_append_val(scores,dist);
    }

    for (n=0; n<seqs->len; n++) {
        bestdist = -1.0;
        bestm = -1;
        for (m=0; m<seqs->len; m++)
            if (g_array_index(scores,double,m) > bestdist) {
                bestdist = g_array_index(scores,double,m);
                bestm = m;
            }
        assert(bestm > -1);
        newdata = calloc(1,sizeof(treest));
        newdata->index = bestm;
        newdata->depth = sqrt(bestdist);
        g_array_index(scores,double,bestm) = -2.0;
        if (v->revcomp) {
            dist1 = 1.0;
            dist1r = 1.0;
            for (m=0; m<n; m++) {
                m1 = g_array_index(added,int,m);
                if (g_array_index(dirs,int,m) == -1) {
                    dist1 *= distance[2*m1][2*bestm+1];
                    dist1r *= distance[2*m1][2*bestm];
                }
                else {
                    dist1 *= distance[2*m1][2*bestm];
                    dist1r *= distance[2*m1][2*bestm+1];
                }
            }
            if (dist1r > dist1)
                newdata->dir = -1;
            else
                newdata->dir = 1;
        }
        else
            newdata->dir = 1;
        m1 = newdata->dir;
        g_array_append_val(dirs,m1);
        g_array_append_val(added,bestm);
        child = g_node_new(newdata);
        g_node_append(root,child);
    }
    
    v->pTree = root;
    g_array_free(scores,TRUE);
    g_array_free(added,TRUE);
    g_array_free(dirs,TRUE);
    
}



 void createStarTree(params *v, GPtrArray *seqs, double depth)
 {
     int i =0;
     GNode *gNode = NULL;
     treest *treedata = NULL;
     GNode *gNodePrev = NULL;
     /* Node for the root of the tree */
     v->pTree = g_node_new(NULL); // Creates the root of the tree

     /* Assign all the clusters seperately */
     for (i=0;i<seqs->len;i++)
     {
         treedata = (treest*)calloc(sizeof(treest), 1);
         treedata->seq = g_ptr_array_index(seqs, i);
         treedata->numofseqs = 1;
         treedata->depth = depth;
         treedata->index = i;
         gNode = g_node_new(treedata);
         if (gNodePrev == NULL)
             v->pTree->children = gNode;
         else
             gNodePrev->next = gNode;
         gNodePrev = gNode;
     } /* end of for loop */
     gNode->next = NULL;
 }


 GNode* get_nth_node(params *v, int value)
 {
     GNode* gNode = NULL;
     treest *treestr= NULL;
     gNode= v->pTree->children;
     while (gNode != NULL)
     {
         treestr = (treest*)gNode->data;
         if (treestr->index == value) return gNode;
         gNode = gNode->next;

     } /* end of while */
     return gNode;
 }


 void merge_nodes(params *v, GNode* gNode3, GNode* gNode1, GNode*gNode2)
 {
     /* add the gNode3 in the place of gNode1 and remove gNode2 */
     GNode *gNode=NULL, *gNodePrev = NULL;
     treest *tst1= NULL;
     gNode = v->pTree->children;
     int num = 0;

     while (gNode != NULL)
     {
         tst1 =(treest*)gNode->data;
         if ((gNode->data == gNode1->data) || (gNode->data == gNode2->data))
         {
             if (num == 1)
             {
                 if (gNodePrev != NULL)
                     gNodePrev->next = gNode3;
                 else
                     v->pTree->children = gNode3;
                 gNode3->next = gNode->next;
                 gNode3->children = gNode1; gNode1->next = gNode2;  gNode2->next = NULL;
                 gNode = gNode3;
                 gNodePrev = gNode;
             }
             else 
             {
                 if (gNodePrev != NULL) 
                 {
                     gNodePrev->next = gNode->next; 
                     gNodePrev = gNodePrev;
                 }
                 else
                 {
                     v->pTree->children = gNode->next;
                 }
                 num++;
             }

         }
         else
             gNodePrev = gNode;

         gNode = gNode->next;
     }
 }

 void createTree(params *v, GPtrArray *seq, double **distance)
 {
     int i = 0,j=0;
     fastaseq *oneseq, *twoseq, *newseq= NULL;
     GNode *gNode = NULL, *gNode1 = NULL, *gNode2 = NULL;
     treest * treedata3, *treedata1, *treedata2;
     int maxi=0, maxj=0;double max = 0.0;
     double dep1, dep2;
     int numi =0, numj=0, flag=0;

     findmaxdist(&max, &maxi, &maxj,distance);
     if (max == 0) return;


     distlen++;

     gNode1 = NULL, gNode2= NULL;
     /* define a new node on the tree which has i and j has children */
     gNode1 =get_nth_node(v, maxi);
     gNode2 = get_nth_node(v, maxj);

     assert(gNode1 != NULL);
     assert(gNode2 != NULL);

     treedata1 = (treest*)gNode1->data;
     treedata2 = (treest*)gNode2->data;
     oneseq = treedata1->seq; numi = treedata1->numofseqs;
     twoseq = treedata2->seq;numj =treedata2->numofseqs;


     newseq = (fastaseq*)calloc(sizeof(fastaseq), 1);
     newseq->seq = g_string_new("");
     newseq->header = g_string_new("");
     newseq->header = g_string_append(newseq->header, oneseq->header->str);
     newseq->header = g_string_append(newseq->header, twoseq->header->str);

     treedata3 = (treest*)calloc(sizeof(treest), 1);
     treedata3->seq = newseq;
     treedata3->numofseqs = treedata1->numofseqs + treedata2->numofseqs;
     treedata3->depth = 0;
     treedata3->index = distlen -1;

     int done = 0;
     dep1 = dep2 = 1.0;
     findLeafDist(gNode1, v->pTree, &dep1, &flag, &done);

     flag = 0; done = 0;
     findLeafDist(gNode2, v->pTree, &dep2, &flag, &done);

     /* assign a distance of sqrt(max) for each leaf node from its ancestor */
     treedata1->depth = sqrt(max)/dep1;

     if (treedata1->depth > 0.99) // FIXME what's appropriate?
         treedata1->depth = 0.99;
     if (treedata1->depth < 0.01)
         treedata1->depth = 0.01;

     treedata2->depth = sqrt(max)/dep2;
     if (treedata2->depth > 0.99)
         treedata2->depth = 0.99;
     if (treedata2->depth < 0.01)
         treedata2->depth = 0.01;


     /* Add this tree data to a new node */
     gNode = g_node_new(treedata3);
     merge_nodes(v, gNode, gNode1, gNode2);

     for (i=0;i< distlen-1; i++)
     {
         double val = 0.0;
         if ((i ==maxi) || (i ==maxj))
             val = 0;
         else
         {
             /* Geometric mean of other nodes to the new non-leaf node created */
             val = sqrt(distance[i][maxi] * distance[i][maxj]);
         }
         distance[i][distlen-1] = val;
         distance[distlen-1][i] = val;
     }
     for (i=0;i<distlen-1;i++)
         for (j=0;j<distlen-1;j++)
             if ((i==maxi) || (j==maxj)) {
                 distance[i][j] = 0;
                 distance[j][i] = 0;
             }

     createTree(v, seq,distance);
 }


void fixdepths(GNode *tree) 
{
    GNode *child;
    child = tree->children;
    while (child) {
        if (((treest*)(child->data))->depth < 0.01)
            ((treest*)(child->data))->depth = 0.01;
        if (((treest*)(child->data))->depth > 0.99)
            ((treest*)(child->data))->depth = 0.99;
        fixdepths(child);
        child = child->next;
    }
}

    

 void findLeafDist(GNode * gN, GNode * gNode, double *dist, int *flag, int*done)
 {
     GNode *child;
     treest* tst, * tst1;
     if (gN == NULL)
         return;
     tst1 = (treest*)gN->data;
     child=gNode->children;
     while (child != NULL) {
         tst = (treest*)child->data;
         if ((*flag == 0) && (tst->index == tst1->index)   )
         {
            *flag = 1;
         }

         if (child->children != NULL)
         {   
             if (*flag == 1)
             {   
                 if (tst->depth != 0)
                     *dist = *dist * tst->depth;
             }
             findLeafDist(gN, child, dist, flag, done);
             if (*done == 1) return;
         }
         else
         {  
             if (*flag == 1)
             {
                 if (tst->depth != 0)
                     *dist = *dist * tst->depth; 
                 *done = 1; return;
             }
         }

         child = child->next;
     }

 }


 void get_leaves(GNode *ptree, GArray **leaves)
 {
     /* assume *leaves is already allocated and empty */

     GNode *child;

     if (ptree->children == NULL) {
         g_array_append_val((*leaves),((treest *)(ptree->data))->index);
     }
     else {
         child = ptree->children;
         while (child) {
             get_leaves(child,leaves);
             child = child->next;
         }
     }
 }


 void add_leaves_to_subtree(params *v, GNode **subtree)
 {
     GArray *leaves;
     GNode *child;

     if ((*subtree)->children) {
         leaves = g_array_new(TRUE,TRUE,sizeof(int));
         get_leaves((*subtree),&leaves);
         ((treest *) (*subtree)->data)->leaves = leaves;
         child = (*subtree)->children;
         while (child) {
             if (child->children)
                 add_leaves_to_subtree(v,&child);
             child = child->next;
         }
     }
 }




 double getTreeProb_rec(params *v, GNode *ptree, seq_frag *frag1, seq_frag *frag2, int index1, int index2, int nancbase) 
 {
     /* recursive function: implement
      * P(nucleotides|T) =  prod_children T(child|ancbase)
      * where T(child|alpha) = T(nuc|alpha) if child is leaf,
      * = sum_beta T(beta|alpha) getTreeProb_rec(child|beta) if child is node
      *
      * (calling function will do sum over ancbase)
      * (transition probabilities for leaves are stored in frag, and
      * transition matrices for nodes are stored in tree)
      *
      */
     int nuc, ns;
     double prob, tmpprob;
     GNode *child;

     prob = 1.0;
     child = ptree->children;
     while (child) {
         if (child->children == NULL) {
             ns = ((treest *)(child->data))->index;
             if (frag1->seq[ns])
                 prob *= frag1->transprob[ns][index1*4+nancbase];
             else  // assume it MUST be in frag2!
                 prob *= frag2->transprob[ns][index2*4+nancbase];
         }
         else {
             /* this is an internal node: for each value B of this node,
              * calculate T(B|ancbase) * prob(tree below B)
              */
             tmpprob = 0.0;

             for (nuc = 0; nuc<4; nuc++)
                 tmpprob += (((treest *)(child->data))->transmat)[nuc*4+nancbase] * getTreeProb_rec(v,child,frag1,frag2,index1,index2,nuc);

             prob *= tmpprob;
         }
         child = child->next;
     }
    
     return prob;
     
}




double getTreeProb(params *v, GNode *ptree, seq_frag *frag1,
                     seq_frag *frag2, int index1, int index2) 
{
    double prob;
    int nuc;
    
    prob = 0.0;

    for (nuc=0; nuc<4; nuc++)
        prob += v->singlesite[nuc]*getTreeProb_rec(v,ptree,frag1,frag2,index1,index2,nuc);
/*    
    if ((index1==0) && ((index2==0)||(frag2==NULL)))
        printf("_________________________\n");
*/    
    return prob;
}



